﻿//---------------------------------------------------------------------
//	グラフィック･イコライザ
//
//	マイコン側のプログラム： 	IO_F446_GraphicEqulizer
//		ボーレート：9600 baud
//
//		作成者：三上直樹，2020/02/11 作成，©三上直樹 2020
//---------------------------------------------------------------------

using System;
using System.Drawing;
using System.Windows.Forms;

using System.IO.Ports;
using ComPortComboBox;
using Mikami.WindowsApplication;
using Mikami;

using static System.Math;

namespace F446_GraphicEqualizer
{
	public partial class Form1 : Form
	{
		private const float F_LOWEST_  = 50.0f;		// 遮断周波数の最小値
		private const float F_HIGHEST_ = 20000.0f;	// 遮断周波数の最大値
		private const int	BAUD_RATE_ = 9600;		// ボー･レート
		private const float	SAMPLING_FRQ_ = 44100;	// 標本化周波数, 44.1 kHz
		private const int	BANDS_ = 9;				// フィルタ数
		private const double F0_MIN_ = 62.5;		// 中心周波数の最小値
		private const double Q_VAL_ = 1.0/1.414214;	// 1.0/Sqrt(2);

        private readonly float dpiFactor_;			// DPI 調整のための係数
		private readonly BiquadGrEq[] filters_ = new BiquadGrEq[BANDS_];
		private readonly TrackBar[] bar_ = new TrackBar[BANDS_];

		private FrqResponsePlotter plotter_;
		private bool	paintOk_;

		public Form1()
		{
			InitializeComponent();

            dpiFactor_ = (this.CreateGraphics().DpiX)/96.0f;

			GrEqDefinition();	// グラフィック･イコライザで使うフィルタの定義
			TrackBarGerator();	// TrackBar 生成

			plotter_ = new FrqResponsePlotter(
						   pictureBox1,		// 表示される PictureBox
						   F_LOWEST_,		// 100 Hz から表示
						   F_HIGHEST_,		// 20 kHz まで表示
						   -24,				// -24 dB から表示
						   24,				// 24 dB まで表示
						   SAMPLING_FRQ_,	// 標本化周波数
						   dpiFactor_);
			paintOk_ = false;

			CommonProc.Disable(button2, panel1);
			// ComboBox の Items の内容を作成し，一番大きな
			// 番号のポートを ComboBox1 の Text に表示する
			ComPort.UpdateComboBoxItems(comboBox1);

			FormCloseConfirm.Set(this);		// 終了時に確認できるようにする

			// DPI が変化した場合の ComboBox の位置の補正
			comboBox1.Location = new Point(comboBox1.Location.X,
										   ClientSize.Height - comboBox1.Height
										   + comboBox1.Margin.Top/2);
		}

		// "開く" または "閉じる" ボタンのクリック
		private void button1_Click(object sender, EventArgs e)
		{
			if (button1.Text == "OPEN")
            {
				if (!SerialPortOpen())
				{
					toolStripStatusLabel1.Text = "必要な COM ポートを開けません．";
					return;
				}
				button1.Text = "CLOSE";
				CommonProc.Enable(button2, panel1);
				comboBox1.Enabled = false;
				foreach(var tb in bar_) tb.Enabled = true;
				toolStripStatusLabel1.Text = "COM ポートを開きました．";
				SendTo("GrEq");			// マイコン側へ "ACK" の転送を要求
				timer2.Enabled = true;	// timer2 を有効化
				radioButton1.Checked = true;
				paintOk_ = true;
            }
            else
            {
                SerialPortClose();
                button1.Text = "OPEN";
				CommonProc.Disable(panel1);
				CommonProc.Do(c => ((RadioButton)c).Checked = false,
									radioButton1, radioButton2);

				foreach(var tb in bar_) tb.Enabled = false;
				CommonProc.Enable(button2, comboBox1);
				toolStripStatusLabel1.Text = "COM ポートを閉じました．";
				paintOk_ = false;
            }
			pictureBox1.Invalidate();
		}

		// 終了
		private void button2_Click(object sender, EventArgs e)
		{
			Close();
		}

		// フラットな周波数特性に戻す
		private void button3_Click(object sender, EventArgs e)
		{
			for (int n=0; n<BANDS_; n++)
				filters_[n].SetCoefficients(0.0);
			pictureBox1.Invalidate();
			for (int n=0; n<BANDS_; n++) bar_[n].Value = 0;
			SendTo("FLAT");
		}

		// フィルタ処理を有効/無効にする
		private void radioButton1_Click(object sender, EventArgs e)
		{
			if ((RadioButton)sender == radioButton1) SendTo("ACTIVE");
			else									 SendTo("THROUGH");
		}

		// 終了時にシリアルポートがオープンであれば，これをクローズする
		private void Form1_FormClosing(object sender, FormClosingEventArgs e)
		{
			if (serialPort1.IsOpen) serialPort1.Close();
		}

		// 目盛値，軸のタイトルの描画
		private void Form1_Paint(object sender, PaintEventArgs e)
		{
			plotter_.DrawCharacters(e.Graphics);
		}

		// 周波数特性のカーブ，グラフの目盛線などの描画
		private void pictureBox1_Paint(object sender, PaintEventArgs e)
		{
			plotter_.DrawAxis(e.Graphics);	// 目盛線の描画
			if (!paintOk_) return;

			plotter_.DrawCurve(e.Graphics, Pens.Blue, GetResponse);
		}

		// データ受信イベントに対応するハンドラ
		private void serialPort1_DataReceived(object sender, SerialDataReceivedEventArgs e)
		{
            if (!serialPort1.IsOpen) return;

			try
			{
				string data = serialPort1.ReadLine();
				if (data == "ACK")
				{
					Invoke(() => timer2.Enabled = false);   // timer2 を無効化
					Invoke(() => toolStripStatusLabel1.Text = "マイコン側の準備ができました．");
					SendTo("ACTIVE");		// フィルタ処理を有効にする指令
				}
			}
			catch (Exception ex)
			{
				ExectionChatch(ex);	
			}
		}

		// WinProc() の中で　ComPort.UpdateComboBoxItems() を呼び出すと
		// 例外が発生するので，ここで呼び出す．
		private void timer1_Tick(object sender, EventArgs e)
		{
			ComPort.UpdateComboBoxItems(comboBox1);
			timer1.Enabled = false;		// timer1 を無効化
		}

		// マイコンの準備ができていない（"ENQ" に対して "ACK" が返ってこない）場合
		private void timer2_Tick(object sender, EventArgs e)
		{
			toolStripStatusLabel1.Text = "マイコン側の準備ができていません．";
			if (serialPort1.IsOpen) serialPort1.Close();	// シリアルポートのクローズ処理

			CommonProc.Disable(panel1);
			foreach(var tb in bar_) tb.Enabled = false;
			paintOk_ = false;
			pictureBox1.Invalidate();

			comboBox1.Enabled = true;
			button1.Text = "OPEN";
		}
		// イベントハンドラはここまで
		//----------------------------------------------------------------------------

		// 以下の Invoke(Action action) を定義しておけばプログラムの記述が楽になる
		//			たとえば，
		//		Invoke(new Action(() => label1.Text = "文字列"));
		//			と書く代わりに
		//		Invoke(() => label1.Text = "文字列");
		//			のように書ける
		private void Invoke(Action action) => base.Invoke(action);

		// 異なる DPI への対応（expression-bodied function）
		public int Dpi(float x) => (int)Round(x*dpiFactor_);
	}
}
