﻿//-------------------------------------------------------------------------
//	遮断周波数可変 LPF/HPF
//		遮断周波数変更指令は周波数特性表示画面のカーソルを移動することで送る
//
//	マイコン側のプログラム： IO_F446_LpfHpf_Variable
//		ボーレート：9600 baud
//
//		作成者：三上直樹，2020/02/13 作成，©三上直樹 2020
//-------------------------------------------------------------------------

using System;
using System.Drawing;
using System.Windows.Forms;

using System.IO.Ports;
using ComPortComboBox;
using Mikami.WindowsApplication;
using Mikami;

using static System.Math;


namespace F446_LPF_HPF_Variable
{
	public partial class Form1 : Form
	{
		private const float F_LOWEST_ = 100.0f;		// 遮断周波数の最小値
		private const float F_HIGHEST_ = 20000.0f;	// 遮断周波数の最大値
		private const int	BAUD_RATE_ = 9600;		// ボー･レート
		private const float	SAMPLING_FRQ_ = 44100;	// 標本化周波数, 44.1 kHz
		private const int	ORDER_ = 10;			// フィルタの次数

        private readonly float	dpiFactor_;			// DPI 調整のための係数
		private readonly Pen	cursor_;			// 遮断周波数のカーソル用 Pen

		private FrqResponsePlotter plotter_;
		private BiliearDesign iir_ = new BiliearDesign();
		private BiliearDesign.Type pb_;

		private bool	paintOk_;
		private float	fC_;			// 遮断周波数
		private bool	mouseDown_;		// マウスボタンを押した状態で true

		public Form1()
		{
			InitializeComponent();

            dpiFactor_ = (this.CreateGraphics().DpiX)/96.0f;

			plotter_ = new FrqResponsePlotter(
						   pictureBox1,		// 表示される PictureBox
						   F_LOWEST_,		// 100 Hz から表示
						   F_HIGHEST_,		// 20 kHz まで表示
						   -60,				// -60 dB まで表示
						   SAMPLING_FRQ_,	// 標本化周波数
						   dpiFactor_);
			paintOk_ = false;
			pb_ = BiliearDesign.Type.LPF;	// 通過域のタイプの初期設定

			cursor_ = Pens.Magenta;	// 遮断周波数のカーソル用 Pen の色はマゼンタ
			mouseDown_ = false;		// マウスボタンは押し下げていない

			CommonProc.Disable(button2, label1, groupBox1, panel1);
			// ComboBox の Items の内容を作成し，一番大きな
			// 番号のポートを ComboBox1 の Text に表示する
			ComPort.UpdateComboBoxItems(comboBox1);

			// フィルタの設計，遮断周波数： 1 kHz
			fC_ = 1000;
			iir_.Design(ORDER_, pb_, SAMPLING_FRQ_, fC_);

			// マウスのイベントハンドラを登録
			pictureBox1.MouseDown += (s, e) => mouseDown_ = true;
			pictureBox1.MouseMove += MyMouseMove;
			pictureBox1.MouseUp += (s, e) => mouseDown_ = false;

			FormCloseConfirm.Set(this);		// 終了時に確認できるようにする

			// DPI が変化した場合の ComboBox の位置の補正
			comboBox1.Location = new Point(comboBox1.Location.X,
										   ClientSize.Height - comboBox1.Height
										   + comboBox1.Margin.Top/2);
		}

		// "OPEN" または "CLOSE" ボタンのクリック
		private void button1_Click(object sender, EventArgs e)
		{
			if (button1.Text == "OPEN")
            {
				if (!SerialPortOpen())
				{
					toolStripStatusLabel1.Text = "必要な COM ポートを開けません．";
					return;
				}
				button1.Text = "CLOSE";
				CommonProc.Enable(button2, label1, groupBox1, panel1);
				comboBox1.Enabled = false;
				label1.Text = "遮断周波数：" + (fC_/1000).ToString() + " kHz";
				toolStripStatusLabel1.Text = "COM ポートを開きました．";
				SendTo("VrFc");			// マイコン側へ "ACK" の転送を要求
				timer2.Enabled = true;	// timer2 を有効化
				radioButton3.Checked = true;
				paintOk_ = true;
            }
            else
            {
                SerialPortClose();
                button1.Text = "OPEN";
				CommonProc.Disable(label1, groupBox1, panel1);
				CommonProc.Do(c => ((RadioButton)c).Checked = false,
									radioButton3, radioButton4);
				CommonProc.Enable(button2, comboBox1);
				toolStripStatusLabel1.Text = "COM ポートを閉じました．";
				paintOk_ = false;
            }
			pictureBox1.Invalidate();
		}

		// 終了
		private void button2_Click(object sender, EventArgs e)
		{
			Close();
		}

		// LPF と HPF の切り替え
		private void radioButton1_Click(object sender, EventArgs e)
		{
			pb_ = radioButton1.Checked ? BiliearDesign.Type.LPF : BiliearDesign.Type.HPF;
			RedesignDisplay();
			if (pb_ == BiliearDesign.Type.LPF) SendInt("L", fC_);
			else							   SendInt("H", fC_);

		}

		// フィルタ処理を有効/無効にする
		private void radioButton3_Click(object sender, EventArgs e)
		{
			if ((RadioButton)sender == radioButton3) SendTo("ACTIVE");
			else									 SendTo("THROUGH");
		}

		// 終了時に シリアルポートがオープンであれば，これをクローズする
		private void Form1_FormClosing(object sender, FormClosingEventArgs e)
		{
			if (serialPort1.IsOpen) serialPort1.Close();
		}

		// 目盛値，軸のタイトルの描画
		private void Form1_Paint(object sender, PaintEventArgs e)
		{
			plotter_.DrawCharacters(e.Graphics);
		}

		// 周波数特性のカーブ，グラフの目盛線などの描画
		private void pictureBox1_Paint(object sender, PaintEventArgs e)
		{
			plotter_.DrawAxis(e.Graphics);	// 目盛線の描画
			if (!paintOk_) return;

			// 遮断周波数に対応するカーソルの描画
			plotter_.DrawVerticalLine(e.Graphics, cursor_, fC_);
			// 周波数特性のカーブの描画
			plotter_.DrawCurve(e.Graphics, Pens.Blue, iir_.GetResponse);
		}

		// データ受信イベントに対応するハンドラ
		private void serialPort1_DataReceived(object sender, SerialDataReceivedEventArgs e)
		{
            if (!serialPort1.IsOpen) return;

			try
			{
				string data = serialPort1.ReadLine();
				if (data == "ACK")
				{　
					Invoke(() => timer2.Enabled = false);   // timer2 を無効化
					Invoke(() => toolStripStatusLabel1.Text = "マイコン側の準備ができました．");
					SendTo("ACTIVE");	// フィルタ処理を有効にする指令
					SendInt("L", fC_);	// 最初は遮断周波数 1 kHz の LFP
				}
			}
			catch (Exception ex)
			{
				ExectionChatch(ex);	
			}
		}

		// WinProc() の中で　ComPort.UpdateComboBoxItems() を呼び出すと
		// 例外が発生するので，ここで呼び出す．
		private void timer1_Tick(object sender, EventArgs e)
		{
			ComPort.UpdateComboBoxItems(comboBox1);
			timer1.Enabled = false;		// timer1 を無効化
		}

		// マイコンの準備ができていない（"ENQ" に対して "ACK" が返ってこない）場合
		private void timer2_Tick(object sender, EventArgs e)
		{
			toolStripStatusLabel1.Text = "マイコン側の準備ができていません．";
			if (serialPort1.IsOpen) serialPort1.Close();	// シリアルポートのクローズ処理

			CommonProc.Disable(label1, groupBox1, panel1);
			paintOk_ = false;
			pictureBox1.Invalidate();

			comboBox1.Enabled = true;
			button1.Text = "OPEN";
		}
		// イベントハンドラはここまで，
		// ただしマウスに関するイベントハンドラは MouseOperations.cs にもある
		//----------------------------------------------------------------------------

		// 以下の Invoke(Action action) を定義しておけばプログラムの記述が楽になる
		//			たとえば，
		//		Invoke(new Action(() => label1.Text = "文字列"));
		//			と書く代わりに
		//		Invoke(() => label1.Text = "文字列");
		//			のように書ける
		private void Invoke(Action action) => base.Invoke(action);

		// 異なる DPI への対応（expression-bodied function）
		public int Dpi(float x) => (int)Round(x*dpiFactor_);
	}
}
