﻿//-----------------------------------------------------------------------------
//	シリアルポートを使った Nucleo-F446 のコントロールの例
//		マイコン側に "ON" または "OFF" を送る
//
//		マイコン側のプログラム： IO_GuiCtrl_LED_ColorSW
//		ボーレート：115,200 baud
//
//		作成者：三上直樹，2020/02/10 作成，©三上直樹 2020
//-----------------------------------------------------------------------------

using System;
using System.Text;
using System.Windows.Forms;

using System.IO.Ports;
using ComPortComboBox;
using Mikami.WindowsApplication;

namespace F446_OnOff
{
	public partial class Form1 : Form
	{
		private const int BAUD_RATE_ = 115200;

		public Form1()
		{
			InitializeComponent();

			panel1.Enabled = false;
			// ComboBox の Items の内容を作成し，一番大きな
			// 番号のポートを ComboBox1 の Text に表示する
			ComPort.UpdateComboBoxItems(comboBox1);

			FormCloseConfirm.Set(this);		// 終了時に終わってもよいか確認できるようにする

			toolStripStatusLabel1.Text = "｢接続｣ ボタンをクリックしてください．";
			// 通信で日本語も使う場合は次の指定が必要
			serialPort1.Encoding = Encoding.GetEncoding("utf-8");	
		}

		// "接続" または "切断" ボタンのクリック
		private void button1_Click(object sender, EventArgs e)
		{
			if (button1.Text == "接続")
            {
				if (!SerialPortOpen())
				{
					toolStripStatusLabel1.Text = "必要な COM ポートをオープンできません．";
					return;
				}
				button1.Text = "切断";
				panel1.Enabled = true;
				button2.Enabled = false;
				toolStripStatusLabel1.Text = "COM ポートをオープンしました．";
				SendTo("ENQ");			// マイコンへ "ACK" の転送を要求
				timer2.Enabled = true;  // timer2 を有効化
				radioButton1.Checked = true;
			}
            else
            {
                SerialPortClose();
                button1.Text = "接続";
				panel1.Enabled = false;
				CommonProc.Do(c => ((RadioButton)c).Checked = false,
									radioButton1, radioButton2);
				button2.Enabled = true;
				toolStripStatusLabel1.Text = "COM ポートをクローズしました．";
            }
		}

		// 終了
		private void button2_Click(object sender, EventArgs e)
		{
			Close();
		}

		// 処理を有効/無効にするための指令を送信する
		private void radioButton1_Click(object sender, EventArgs e)
		{
			if ((RadioButton)sender == radioButton1) SendTo("ON");
			else									 SendTo("OFF");
		}

		// 終了時に シリアルポートがオープンであれば，これをクローズする
		private void Form1_FormClosing(object sender, FormClosingEventArgs e)
		{
			if (serialPort1.IsOpen) serialPort1.Close();
		}

		// データ受信イベントに対応するハンドラ
		private void serialPort1_DataReceived(object sender, SerialDataReceivedEventArgs e)
		{
            if (!serialPort1.IsOpen) return;
			try
			{
				string data = serialPort1.ReadLine();
				if (data == "ACK")
				{
					Invoke(() => timer2.Enabled = false);	// timer2 を無効化
					Invoke(() => toolStripStatusLabel1.Text = "マイコン側の準備ができました．");
					SendTo("ON");
				}
				else	// 異なるスレッドの GUI に受信データを書き込む
				{
					if (data[0] == 'L')		// label1 に表示する
						Invoke(() => label1.Text = data.Substring(1));

					if (data[0] == 'M')		// toolStripStatusLabel1 に表示する
						Invoke(() => toolStripStatusLabel1.Text = data.Substring(1));
				}
			}
			catch (Exception ex)
			{
				ExectionChatch(ex);	
			}
		}

		// WinProc() の中で　ComPort.UpdateComboBoxItems() を呼び出すと
		// 例外が発生するので，ここで呼び出す．
		private void timer1_Tick(object sender, EventArgs e)
		{
			ComPort.UpdateComboBoxItems(comboBox1);
			timer1.Enabled = false;		// timer1 を無効化
		}

		// マイコンの準備ができていない（"ENQ" に対して "ACK" が返ってこない）場合
		private void timer2_Tick(object sender, EventArgs e)
		{
			toolStripStatusLabel1.Text = "マイコン側の準備ができていません．";
			if (serialPort1.IsOpen) serialPort1.Close();	// シリアルポートのクローズ処理
			button1.Text = "接続";
		}

		// 以下の Invoke(Action action) を定義しておけばプログラムの記述が楽になる
		//			たとえば，
		//		Invoke(new Action(() => label1.Text = "文字列"));
		//			と書く代わりに
		//		Invoke(() => label1.Text = "文字列");
		//			のように書ける
		private void Invoke(Action action) => base.Invoke(action);
	}
}
