﻿//--------------------------------------------------------------
//	スペクトログラムで使うメソッド，描画に関係するもの
//
//		作成者：三上直樹，2020/02/16 作成，©三上直樹 2020
//--------------------------------------------------------------

using System;
using System.Drawing;
using System.Windows.Forms;

using System.Drawing.Imaging;	// 高度な GDI+ イメージング機能

namespace F446_Spectrogram
{
	public partial class Form1 : Form
	{
		private void DrawGraph(Graphics gr)
		{
			gr.TranslateTransform(orgX_, orgY_);	// 座標軸の原点の移動
			Font myFont = new Font("Meiryo UI", 10);

			// 座標軸の描画（横）
			gr.DrawLine(Pens.Black, 0, 5, N_T_, 5);
			float oneSec = 62.5f;	// 1秒に対応するピクセル数
			for (int n=0; n<=(int)(N_T_/oneSec); n++)
			{
				gr.DrawLine(Pens.Black, n*oneSec, 1, n*oneSec, 5);
				gr.DrawString(n.ToString(), myFont, Brushes.Black, n*oneSec-6, 10);
			}
			gr.DrawString("時間 [s]", myFont, Brushes.Black, N_T_/2-30, 28);

			// 座標軸の描画（縦）
			gr.DrawLine(Pens.Black, -5, 0, -5, -N_Y_);
			for (int n=0; n<=8; n++)
				gr.DrawLine(Pens.Black, -5, -n*32, -1, -n*32);
			for (int n=0; n<=4; n++)
				gr.DrawString((n*2).ToString(), myFont, Brushes.Black, -20, -n*64-8);
			gr.RotateTransform(-90);
			gr.DrawString("周波数 [kHz]", myFont, Brushes.Black, 88, -44);
			gr.ResetTransform();

			// スペクトル１次元表示の座標軸
			gr.TranslateTransform(pictureBox2.Location.X,
								  pictureBox2.Location.Y + pictureBox2.Size.Height);
			// 座標軸の描画（横）
			gr.DrawLine(Pens.Black, 0, 5, N_Y_, 5);
			for (int n=0; n<=8; n++)
				gr.DrawLine(Pens.Black, n*32, 5, n*32, 1);
			for (int n=0; n<=4; n++)
				if (radioButton7.Checked) gr.DrawString(n.ToString(), myFont, Brushes.Black, n*64-6, 10);
				else					   gr.DrawString((n*2).ToString(), myFont, Brushes.Black, n*64-6, 10);
			gr.DrawString("周波数 [kHz]", myFont, Brushes.Black, 88, 28);

			// 座標軸の描画（縦）
			Y_Axis(gr, myFont);
			gr.RotateTransform(-90);
			gr.DrawString("相対強度 [dB]", myFont, Brushes.Black, 40, -54);
			gr.ResetTransform();

			// 色と dB の関係
			gr.TranslateTransform(pictureBox1.Location.X + pictureBox1.Width + Dpi(34),
								  pictureBox2.Location.Y + pictureBox2.Size.Height);
			Y_Axis(gr, myFont);

			int x1 = Dpi(4);
			int x2 = x1 + Dpi(10);
			float yF = 1.0f/(Y20DB_*4);
			Pen myPen = new Pen(Color.Transparent, 1);
			for (int n=0; n<=Y20DB_*4; n++)
			{
				myPen.Color = HueScale(n*yF);
				gr.DrawLine(myPen, x1, -n, x2, -n);
			}
		}

		// 座標軸の描画（縦）
		private void Y_Axis(Graphics gr, Font myFont)
		{
			gr.DrawLine(Pens.Black, -5, 0, -5, -Y20DB_*4);
			for (int n=0; n<=8; n++)
				gr.DrawLine(Pens.Black, -5, -n*Y20DB_/2, -1, -n*Y20DB_/2);
			gr.DrawString("0", myFont, Brushes.Black, -22, -8);
			for (int n=1; n<=4; n++)
				gr.DrawString((n*20).ToString(), myFont, Brushes.Black, -30, -n*Y20DB_-8);
		}

		// スペクトルの１次元表示
		private void DrawSpectrum1D(Graphics gr, PointF[] db)
		{
			gr.DrawCurve(Pens.Blue, db);
		}

		// 表示用バッファのクリア
		private Color[,] TwoD_Clear()
		{
			for (int j=0; j<N_T_; j++)
				for (int k=0; k<N_Y_; k++)
					twoD_[j, k] = SystemColors.ControlLightLight;
			return twoD_;
		}

		// 色相の違いで表示
		//		0.0 <= x <= 1.0
		//		x > 1 の場合　戻り値： Color.White
		//		x < 0 の場合　戻り値： Color.Blue
		private Color HueScale(float x)
		{
			if (x > 1.0f) return Color.White;
			if (x < 0.0f) return Color.Blue;

			int r = 0;
			int b = 0;

			if (x<0.5f) b = (x<0.33f) ? 255 : -(int)(1500.0f*x) + 750;
			else	    r = (0.67f<x) ? 255 : (int)(1500.0f*x) - 750;
			int g = 255 - (int)(1020.0f*(x - 0.5f)*(x - 0.5f));

			return Color.FromArgb(r, g, b);
		}

		// ビットマップへの高速転送
		//		1 ピクセル当たり 24 ビット割り当て
		Bitmap DrawBitmap(Color[,] twoD)
		{
			int width = twoD.GetLength(0);
			int height = twoD.GetLength(1);
			Bitmap dst = new Bitmap(width, height, PixelFormat.Format24bppRgb);

			// Bitmap のメモリをロックする
			BitmapData dstData = dst.LockBits(new Rectangle(Point.Empty, dst.Size),
											  ImageLockMode.WriteOnly,
											  PixelFormat.Format24bppRgb);
			int myStride = dstData.Stride;

			// プロジェクトのプロパティをアンセーフコード許可にすること
			unsafe
			{
				byte *dstScan0 = (byte*)dstData.Scan0;	// ピクセルデータのポインタを取得

				for (int jy = 0; jy < height; jy++)
					for (int jx = 0; jx < width; jx++)
					{
						int dPos = 3*jx + myStride*jy;
						dstScan0[dPos+2] = twoD[jx, jy].R;	// R
						dstScan0[dPos+1] = twoD[jx, jy].G;	// G
						dstScan0[dPos+0] = twoD[jx, jy].B;	// B
					}
			}
			dst.UnlockBits(dstData);

			return dst;
		}
	}
}
