﻿//---------------------------------------------------------------------------------------------
//	スペクトログラム
//
//		マイコン側のプログラム：IO_F446_Spectrogram
//		ボーレート：460800 baud
//
//		マイコンから受け取るデータ：１つのデータ当たり 0 ～ 10,000 に対応する2文字
//
//		ビルドの際にプロジェクトのプロパティで「アンセーフコードの許可」にチェックすること
//
//		作成者：三上直樹，2020/02/16 作成，©三上直樹 2020
//---------------------------------------------------------------------------------------------

using System;
using System.Drawing;
using System.Windows.Forms;

using System.IO.Ports;
using Mikami.ComPortComboBox;
using Mikami.WindowsApplication;

using static System.Math;

namespace F446_Spectrogram
{
	public partial class Form1 : Form
	{
		private const int BAUD_RATE_ = 460800;	// ボー･レート
		private const int N_Y_ = 257;			// 縦（周波数方向）のピクセル数
		private const int N_T_ = 500;			// 横（時間方向）のピクセル数

		private const float Y_SCALE_ = 2.5f;	// スペクトル一次元表示の縦方向のスケール
		private const int Y20DB_ = (int)(Y_SCALE_*20);
		private const float A1_ = 0.96f;		// 表示の平滑化用 LPF の係数
		private const float B0_ = 1.0f - A1_;	// 表示の平滑化用 LPF の係数

		private readonly int FORM_WIDTH_;		// Form の最初のサイズ
		private readonly float	dpiFactor_;		// DPI 調整のための係数

		private readonly int orgX_;		// スペクトログラムの x 座標の原点
		private readonly int orgY_;		// スペクトログラムの y 座標の原点
		private readonly float orgY2_;	// スペクトル一次元表示の y 座標の原点

		private Color[,] twoD_ = new Color[N_T_, N_Y_];	// スペクトログラムに対応するデータ
		private PointF[] pYn_ = new PointF[N_Y_];		// 一次元スペクトル表示用
		private float[] yM1_ = new float[N_Y_];		// 一次元スペクトル表示用
		private string rxData_;			// 受信データ用バッファ
		private int nData_;				// データ数
		private bool run_;				// RUN のとき true

		// イベントを発生する側のオブジェクトを生成する
		private RxCompletedEvent evt = new RxCompletedEvent();

		public Form1()
		{
			InitializeComponent();

			evt.RxCompleted += OnRxCompleted;	// イベントリストにハンドラを追加する

            dpiFactor_ = (this.CreateGraphics().DpiX)/96.0f;
			FORM_WIDTH_ = Size.Width;

			orgX_ = pictureBox1.Location.X;
			orgY_ = pictureBox1.Location.Y + Dpi(N_Y_);
			orgY2_ = pictureBox2.Size.Height;
			for (int n=0; n<N_Y_; n++) yM1_[n] = orgY2_;

			button2.ForeColor = Color.DodgerBlue;
			CommonProc.Disable(button2, button3, button4, panel1);
			Width = FORM_WIDTH_;
			FormBorderStyle = FormBorderStyle.FixedSingle;

			// ComboBox の Items の内容を作成し，一番大きな番号のポートを
			// ComboBox1 の Text に表示する
			ComPort.UpdateComboBoxItems(comboBox1);

			label4.Text = trackBar1.Value.ToString() + " dB";

			run_ = false;
			TwoD_Clear();
		}

		// "Open" または "Close" ボタンのクリック
		private void button1_Click(object sender, EventArgs e)
		{
			if (button1.Text == "Open")
            {
				if (!serialPortOpen())
				{
					toolStripStatusLabel1.Text = "必要な COM ポートをオープンできません．";
					return;
				}
				button1.Text = "Close";
				CommonProc.Enable(button2, button4, panel1);
				comboBox1.Enabled = false;
				SendTo("Spectrogram");	// マイコンへ "ACK" の転送を要求
				timer2.Enabled = true;	// timer2 を有効化
            }
            else
            {
				serialPort1.Close();	// シリアルポートのクローズ処理
                button1.Text = "Open";
				button2.Enabled = false;
				CommonProc.Disable(button2, panel1);
				comboBox1.Enabled = true;
				toolStripStatusLabel1.Text = "COM ポートをクローズしました．";
            }
		}

		// マイコン側からのデータ取得実行の開始
		private void button2_Click(object sender, EventArgs e)
		{
			run_ = true;
			CommonProc.Disable(button1, button2);
			button3.Enabled = true;
			toolStripStatusLabel1.Text = "RUN";
			RequestXfer("GO");
		}

		// 停止
		private void button3_Click(object sender, EventArgs e)
		{
			run_ = false;
			CommonProc.Enable(button1, button2);
			button3.Enabled = false;
			toolStripStatusLabel1.Text = "STOP";
		}

		// クリア
		private void button4_Click(object sender, EventArgs e)
		{
			pictureBox1.Image = DrawBitmap(TwoD_Clear());
		}

		// フォームを閉じることの確認と，閉じてもよいときに
		// シリアルポートがオープンであれば，これをクローズする
		private void Form1_FormClosing(object sender, FormClosingEventArgs e)
		{
			bool runSave = run_;
			run_ = false;
			DialogResult result =
				MessageBox.Show("閉じてもいいですか？",
								"終了の確認",
								MessageBoxButtons.YesNo,
								MessageBoxIcon.Information);
			if (result == DialogResult.No)
			{
				e.Cancel = true;	// 閉じるのをキャンセルする
				if (runSave)
				{
					run_ = true;
					OnRxCompleted();
				}
				return;
			}

			if (serialPort1.IsOpen) serialPort1.Close();
		}

		// 座標軸等の描画
		private void Form1_Paint(object sender, PaintEventArgs e)
		{
			DrawGraph(e.Graphics);
		}

		// スペクトルの１次元表示
		private void pictureBox2_Paint(object sender, PaintEventArgs e)
		{
			DrawSpectrum1D(e.Graphics, pYn_);
		}

		// レベル調整用トラックバーの値の表示
		private void trackBar1_Scroll(object sender, EventArgs e)
		{
			label4.Text = trackBar1.Value.ToString() + " dB";
		}
/*
		private void trackBar1_Scroll(object sender, EventArgs e)
		{
			label4.Text = trackBar1.Value.ToString() + " dB";
		}
*/
		// 表示する周波数の最大値を変更した場合の対応
		private void radioButton7_CheckedChanged(object sender, EventArgs e)
		{
			Invalidate();
		}
/*		private void radioButton10_CheckedChanged(object sender, EventArgs e)
		{
			Invalidate();
		}
*/
		// データ受信イベントに対応するハンドラ
		private void serialPort1_DataReceived(object sender, SerialDataReceivedEventArgs e)
		{
            if (!serialPort1.IsOpen) return;

			try
			{
				string data = serialPort1.ReadLine();
				if (data == "ACK")
				{
					Invoke(() => timer2.Enabled = false);   // timer2 を無効化
					Invoke(() => toolStripStatusLabel1.Text = "マイコン側の準備ができました．");

					return;
				}
				if (data == "EOT")	// 受信データの終了を検出した場合の処理
				{
					nData_ = rxData_.Length/2;
					evt.Fire();		// 終了を通知するイベントを発生させる
					return;
				}
				// 受信したデータを保存する
				rxData_ = data;
			}
			catch (Exception ex)
			{
				ExectionChatch(ex);
			}
		}

		// evt.Fire() の実行により起動するイベントハンドラ
		// 受信終了に対するイベントハンドラとして使う
		void OnRxCompleted()
		{
			// 表示用バッファのデータを一つ左に移動
			for (int j=1; j<N_T_; j++)
				for (int k=0; k<N_Y_; k++)
					twoD_[j-1, k] = twoD_[j, k];

			// 表示用バッファの更新
			int dx = radioButton8.Checked ? 1 : 2;	// 1: 8 kHz まで表示，2: 4 kHz まで表示
			for (int n=0; n<nData_; n++)
			{
				float x = C10000ToFloat(rxData_.Substring(2*n, 2));

				float xn = orgY2_ -1 -x*80.0f*Y_SCALE_;
				float yn = B0_*xn + A1_*yM1_[n];
				if (radioButton5.Checked) yn = (yn < xn) ? yn : xn;
				pYn_[n] = new PointF(dx*n, yn);
				yM1_[n] = yn;

				twoD_[N_T_-1, nData_-n-1] = HueScale(x);
			}
			// 直流分は無視するための処理
			pYn_[0] = pYn_[1] = pYn_[2];
			twoD_[N_T_-1, nData_-1] = twoD_[N_T_-1, nData_-2] = twoD_[N_T_-1, nData_-3];

			pictureBox1.Image = DrawBitmap(twoD_);

			pictureBox2.Invalidate();
			
			if (run_) Invoke(() => RequestXfer("GO") );
		}

		// WinProc() の中で　ComPort.UpdateComboBoxItems() を呼び出すと
		// 例外が発生するので，ここで呼び出している
		private void timer1_Tick(object sender, EventArgs e)
		{
			ComPort.UpdateComboBoxItems(comboBox1);
			timer1.Enabled = false;		// timer1 を無効化
		}

		// マイコンの準備ができていない（"ENQ" に対して "ACK" が返ってこない）場合
		private void timer2_Tick(object sender, EventArgs e)
		{
			toolStripStatusLabel1.Text = "マイコン側の準備ができていません．";
			button2.Enabled = false;
			if (serialPort1.IsOpen) serialPort1.Close();	// シリアルポートのクローズ処理
			comboBox1.Enabled = true;
			button1.Text = "Open";
		}

		// イベントハンドラはここまで
		//----------------------------------------------------------------------------

		// 以下の Invoke(Action action) を定義しておけばプログラムの記述が楽になる
		//			たとえば，
		//		Invoke(new Action(() => label1.Text = "文字列"));
		//			と書く代わりに
		//		Invoke(() => label1.Text = "文字列");
		//			のように書ける
		private void Invoke(Action action) => base.Invoke(action);

		// 異なる DPI への対応（expression-bodied function）
		public int Dpi(float x) => (int)Round(x*dpiFactor_);
	}

	// 受信データ終了時にイベントを発生するためのクラス
	class RxCompletedEvent
	{
		public event Action RxCompleted;	// イベントを宣言する

		// 終了時にイベントを発生するメソッド
		public void Fire() { RxCompleted(); }
	}
}
